@@ -29,8 +29,44 @@
 #include <strings.h>
 
 #include "g80_type.h"
+#include "g80_display.h"
 #include "g80_output.h"
 
+static unsigned G80FindLoadVal(const unsigned char *table1)
+{
+    const unsigned char *p = table1;
+    int count;
+    const CARD32 def = 340;
+
+    for(p = table1; *(CARD16*)p != 0xb8ff && p < table1 + 64000; p += 2);
+    if(p == table1 + 64000)
+        return def;
+    p += 2;
+    if(*(CARD32*)p != 0x544942)
+        return def;
+    p += 4;
+    if(*(CARD16*)p != 0x100)
+        return def;
+    p += 2;
+    if(*p != 12)
+        return def;
+    p++;
+    if(*p != 6)
+        return def;
+    p++;
+    count = *p;
+    p += 2;
+    for(; *p != 'A' && count >= 0; count--, p += 6);
+    if(count == -1)
+        return def;
+    p += 4;
+    p = table1 + *(CARD16*)p;
+    p = table1 + *(CARD16*)p;
+    if(p[0] != 0x10 || p[1] != 4 || p[2] != 4 || p[3] != 2)
+        return def;
+    return *(CARD32*)(p + 4) & 0x3ff;
+}
+
 static Bool G80ReadPortMapping(int scrnIndex, G80Ptr pNv)
 {
     unsigned char *table2;
@@ -65,10 +101,11 @@ static Bool G80ReadPortMapping(int scrnIndex, G80Ptr pNv)
         port = (b >> 4) & 0xf;
         or = ffs((b >> 24) & 0xf) - 1;
 
-        if(type < 4 && port != 0xf) {
+        if(type == 0xe) break;
+
+        if(type < 4) {
             switch(type) {
                 case 0: /* CRT */
-                case 1: /* TV */
                     if(pNv->i2cMap[port].dac != -1) {
                         xf86DrvMsg(scrnIndex, X_WARNING,
                                    "DDC routing table corrupt!  DAC %i -> %i "
@@ -77,21 +114,30 @@ static Bool G80ReadPortMapping(int scrnIndex, G80Ptr pNv)
                     }
                     pNv->i2cMap[port].dac = or;
                     break;
+                case 1: /* TV */
+                    /* Ignore TVs */
+                    break;
+
                 case 2: /* TMDS */
-                case 3: /* LVDS */
                     if(pNv->i2cMap[port].sor != -1)
                         xf86DrvMsg(scrnIndex, X_WARNING,
                                    "DDC routing table corrupt!  SOR %i -> %i "
                                    "for port %i\n",
                                    or, pNv->i2cMap[port].sor, port);
                     pNv->i2cMap[port].sor = or;
-                    pNv->i2cMap[port].panelType = (type == 2) ? TMDS : LVDS;
+                    break;
+
+                case 3: /* LVDS */
+                    pNv->lvds.present = TRUE;
+                    pNv->lvds.or = or;
                     break;
             }
         }
     }
 
-    xf86DrvMsg(scrnIndex, X_PROBED, "I2C map:\n");
+    xf86DrvMsg(scrnIndex, X_PROBED, "Connector map:\n");
+    if(pNv->lvds.present)
+        xf86DrvMsg(scrnIndex, X_PROBED, "  [N/A] -> SOR%i (LVDS)\n", pNv->lvds.or);
     for(i = 0; i < 4; i++) {
         if(pNv->i2cMap[i].dac != -1)
             xf86DrvMsg(scrnIndex, X_PROBED, "  Bus %i -> DAC%i\n", i, pNv->i2cMap[i].dac);
@@ -99,6 +145,9 @@ static Bool G80ReadPortMapping(int scrnIndex, G80Ptr pNv)
             xf86DrvMsg(scrnIndex, X_PROBED, "  Bus %i -> SOR%i\n", i, pNv->i2cMap[i].sor);
     }
 
+    pNv->loadVal = G80FindLoadVal(pNv->table1);
+    xf86DrvMsg(scrnIndex, X_PROBED, "Load detection: %d\n", pNv->loadVal);
+
     return TRUE;
 
 fail:
@@ -173,13 +222,6 @@ G80OutputModeValid(xf86OutputPtr output, DisplayModePtr mode)
     return MODE_OK;
 }
 
-Bool
-G80OutputModeFixup(xf86OutputPtr output, DisplayModePtr mode,
-                   DisplayModePtr adjusted_mode)
-{
-    return TRUE;
-}
-
 void
 G80OutputPrepare(xf86OutputPtr output)
 {
@@ -321,23 +363,31 @@ G80CreateOutputs(ScrnInfoPtr pScrn)
         if(pNv->i2cMap[i].dac != -1)
             dac = G80CreateDac(pScrn, pNv->i2cMap[i].dac);
         if(pNv->i2cMap[i].sor != -1)
-            sor = G80CreateSor(pScrn, pNv->i2cMap[i].sor,
-                               pNv->i2cMap[i].panelType);
+            sor = G80CreateSor(pScrn, pNv->i2cMap[i].sor, TMDS);
 
         if(dac) {
             G80OutputPrivPtr pPriv = dac->driver_private;
 
             pPriv->partner = sor;
             pPriv->i2c = i2c;
+            pPriv->scale = G80_SCALE_OFF;
         }
         if(sor) {
             G80OutputPrivPtr pPriv = sor->driver_private;
 
             pPriv->partner = dac;
             pPriv->i2c = i2c;
+            pPriv->scale = G80_SCALE_ASPECT;
         }
     }
 
+    if(pNv->lvds.present) {
+        xf86OutputPtr lvds = G80CreateSor(pScrn, pNv->lvds.or, LVDS);
+        G80OutputPrivPtr pPriv = lvds->driver_private;
+
+        pPriv->scale = G80_SCALE_ASPECT;
+    }
+
     /* For each output, set the crtc and clone masks */
     for(i = 0; i < xf86_config->num_output; i++) {
         xf86OutputPtr output = xf86_config->output[i];
